@@ -21,6 +21,7 @@ def allclose(a, b, atol = 1e-4):
 @pytest.mark.parametrize('qk_dim_head', [64])
 @pytest.mark.parametrize('v_dim_head', [64])
 @pytest.mark.parametrize('float16', [False, True])
+@pytest.mark.parametrize('attn_bias_batch_dim', [False, True])
 def test_output_equal(
     causal,
     mask,
@@ -28,19 +29,21 @@ def test_output_equal(
     seq_len,
     qk_dim_head,
     v_dim_head,
-    float16
+    float16,
+    attn_bias_batch_dim
 ):
+    batch, heads = 4, 8
     dtype, atol = (torch.float16, 1e-1) if float16 else (torch.float32, 1e-4)
 
-    q = torch.randn(4, 8, seq_len, qk_dim_head, dtype = dtype).cuda()
-    k = torch.randn(4, 8, seq_len, qk_dim_head, dtype = dtype).cuda()
-    v = torch.randn(4, 8, seq_len, v_dim_head, dtype = dtype).cuda()
+    q = torch.randn(batch, heads, seq_len, qk_dim_head, dtype = dtype).cuda()
+    k = torch.randn(batch, heads, seq_len, qk_dim_head, dtype = dtype).cuda()
+    v = torch.randn(batch, heads, seq_len, v_dim_head, dtype = dtype).cuda()
 
-    attn_mask = torch.randint(0, 2, (4, seq_len), dtype = torch.bool).cuda() if mask else None
-    bias = torch.randn(8, seq_len, seq_len, dtype = dtype).cuda() if attn_bias else None
+    attn_mask = torch.randint(0, 2, (batch, seq_len), dtype = torch.bool).cuda() if mask else None
+    bias = torch.randn(batch if attn_bias_batch_dim else heads, seq_len, seq_len, dtype = dtype).cuda() if attn_bias else None
 
-    plain_output = plain_cosine_sim_attention(q, k, v, causal = causal, mask = attn_mask, attn_bias = bias)
-    flash_output = flash_cosine_sim_attention(q, k, v, causal = causal, mask = attn_mask, attn_bias = bias)
+    plain_output = plain_cosine_sim_attention(q, k, v, causal = causal, mask = attn_mask, attn_bias = bias, attn_bias_batch_dim = attn_bias_batch_dim)
+    flash_output = flash_cosine_sim_attention(q, k, v, causal = causal, mask = attn_mask, attn_bias = bias, attn_bias_batch_dim = attn_bias_batch_dim)
 
     assert not_nan_or_infs(flash_output)
     assert allclose(plain_output, flash_output, atol = atol)
@@ -51,6 +54,7 @@ def test_output_equal(
 @pytest.mark.parametrize('qk_dim_head', [64])
 @pytest.mark.parametrize('v_dim_head', [64])
 @pytest.mark.parametrize('float16', [False, True])
+@pytest.mark.parametrize('attn_bias_batch_dim', [False, True])
 def test_grad_equal(
     causal,
     mask,
@@ -58,18 +62,20 @@ def test_grad_equal(
     seq_len,
     qk_dim_head,
     v_dim_head,
-    float16
+    float16,
+    attn_bias_batch_dim
 ):
+    batch, heads = 4, 8
     dtype, atol = (torch.float16, 1e-1) if float16 else (torch.float32, 1e-4)
 
-    q = torch.randn(4, 8, seq_len, qk_dim_head).cuda().requires_grad_()
-    k = torch.randn(4, 8, seq_len, qk_dim_head).cuda().requires_grad_()
-    v = torch.randn(4, 8, seq_len, v_dim_head).cuda().requires_grad_()
+    q = torch.randn(batch, heads, seq_len, qk_dim_head).cuda().requires_grad_()
+    k = torch.randn(batch, heads, seq_len, qk_dim_head).cuda().requires_grad_()
+    v = torch.randn(batch, heads, seq_len, v_dim_head).cuda().requires_grad_()
 
-    attn_mask = torch.randint(0, 2, (4, seq_len), dtype = torch.bool).cuda() if mask else None
-    bias = torch.randn(8, seq_len, seq_len).cuda().requires_grad_() if attn_bias else None
+    attn_mask = torch.randint(0, 2, (batch, seq_len), dtype = torch.bool).cuda() if mask else None
+    bias = torch.randn(batch if attn_bias_batch_dim else heads, seq_len, seq_len).cuda().requires_grad_() if attn_bias else None
 
-    plain_output = plain_cosine_sim_attention(q, k, v, causal = causal, mask = attn_mask, attn_bias = bias)
+    plain_output = plain_cosine_sim_attention(q, k, v, causal = causal, mask = attn_mask, attn_bias = bias, attn_bias_batch_dim = attn_bias_batch_dim)
     plain_output.sum().backward()
 
     dq, dk, dv = q.grad, k.grad, v.grad
@@ -81,7 +87,7 @@ def test_grad_equal(
     if attn_bias:
         bias.grad = None
 
-    flash_output = flash_cosine_sim_attention(q, k, v, causal = causal, mask = attn_mask, attn_bias = bias)
+    flash_output = flash_cosine_sim_attention(q, k, v, causal = causal, mask = attn_mask, attn_bias = bias, attn_bias_batch_dim = attn_bias_batch_dim)
     flash_output.sum().backward()
 
     fdq, fdk, fdv = q.grad, k.grad, v.grad
